#!/usr/bin/env python
###############################################################################
#  Magic.py -- generate magic items
###############################################################################

import string
import Dice
import _Treasure

###############################################################################
#  The following is a multi-table comprising the entire random magic system
#  from 2nd Edition AD&D.  Try not to choke on it :-)
###############################################################################

def _typify(row):
    addl = Dice.Roll(row[2])
    return (0, row[1] % addl[1], addl[2])

_animal_control_potion_table = [
    (4, "(Mammal/Marsupial)", 250),
    (4, "(Avian)", 250),
    (4, "(Reptile/Amphibian)", 250),
    (3, "(Fish)", 250),
    (2, "(Mammal/Marsupial/Avian)", 250),
    (2, "(Reptile/Amphibian/Fish)", 250),
    (1, "(Any Type)", 250),
]

_dragon_control_potion_table = [
    (2, "White", 700),
    (2, "Black", 700),
    (3, "Green", 700),
    (2, "Blue", 700),
    (1, "Red", 700),
    (2, "Brass", 700),
    (2, "Copper", 700),
    (1, "Bronze", 700),
    (1, "Silver", 700),
    (1, "Gold", 700),
    (2, "Evil", 700),
    (1, "Good", 700),
]

_giant_control_potion_table = [
    (5, "Hill", 600),
    (4, "Stone", 600),
    (4, "Frost", 600),
    (4, "Fire", 600),
    (2, "Cloud", 600),
    (1, "Storm", 600),
]

_giant_strength_potion_table = [
    (6, "Hill", 550),
    (4, "Stone", 550),
    (4, "Frost", 550),
    (3, "Fire", 550),
    (2, "Cloud", 550),
    (1, "Storm", 550),
]

_human_control_potion_table = [
    (2, "Potion of Dwarf Control", 500),
    (2, "Potion of Elf/Half-Elf Control", 500),
    (2, "Potion of Gnome Control", 500),
    (2, "Potion of Halfling Control", 500),
    (2, "Potion of Half-Orc Control", 500),
    (6, "Potion of Human Control", 500),
    (3, "Potion of Humanoid Control", 500),
    (1, "Potion of Elf, Half-Elf, and Human Control", 500),
]

_elemental_invulnerability_table = [
    (1, "Air", 500),
    (1, "Earth", 500),
    (1, "Fire", 500),
    (1, "Water", 500),
]

_undead_control_potion_table = [
    (1, "Ghast", 700),
    (1, "Ghost", 700),
    (1, "Ghoul", 700),
    (1, "Shadow", 700),
    (1, "Skeleton", 700),
    (1, "Spectre", 700),
    (1, "Wight", 700),
    (1, "Wraith", 700),
    (1, "Vampire", 700),
    (1, "Zombie", 700),
]

_water_breathing_potion_table = [
    (3, "2", 400),
    (1, "4", 800),
]

_potion_table = [
    (1, "Potion of %s Control", 
        _animal_control_potion_table, _typify),
    (1, "Potion of Clairaudience", 250),
    (1, "Potion of Clairvoyance", 300),
    (1, "Potion of Climbing", 300),
    (2, "Potion of Delusion", 0),
    (1, "Potion of Diminution", 300),
    (1, "Potion of %s Dragon Control", 
        _dragon_control_potion_table, _typify),
    (1, "Elixir of Health", 350),
    (2, "Elixir of Madness", 0),
    (1, "Elixir of Youth", 500),
    (1, "Potion of ESP", 500),
    (2, "Potion of Extra-healing", 400),
    (1, "Potion of Fire Breath", 400),
    (1, "Potion of Fire Resistance", 250),
    (1, "Potion of Flying", 500),
    (1, "Potion of Gaseous Form", 300),
    (1, "Potion of %s Giant Control",
        _giant_control_potion_table, _typify),
    (1, "Potion of %s Giant Strength [Warrior]",
        _giant_strength_potion_table, _typify),
    (1, "Potion of Growth", 250),
    (2, "Potion of Healing", 200),
    (1, "Potion of Heroism [Warrior]", 300),
    (1, "Potion of Human Control", _human_control_potion_table),
    (1, "Potion of Invisibility", 250),
    (1, "Potion of Invulnerability [Warrior]", 350),
    (1, "Potion of Levitation", 250),
    (1, "Potion of Longevity", 500),
    (1, "Oil of Acid Resistance", 500),
    (1, "Oil of Disenchantment", 750),
    (1, "Oil of %s Elemental Invulnerability",
        _elemental_invulnerability_table, _typify),
    (1, "Oil of Etherealness", 600),
    (1, "Oil of Fiery Burning", 500),
    (1, "Oil of Fumbling", 0),
    (1, "Oil of Impact", 750),
    (1, "Oil of Slipperiness", 400),
    (1, "Oil of Timelessness", 500),
    (1, "Philter of Glibness", 500),
    (1, "Philter of Love", 200),
    (1, "Philter of Persuasiveness", 400),
    (1, "Philter of Stammering and Stuttering", 0),
    (1, "Potion of Plant Control", 250),
    (2, "Poison", 0),
    (1, "Potion of Polymorph Self", 200),
    (1, "Potion of Rainbow Hues", 200),
    (1, "Potion of Speed", 200),
    (2, "Potion of Super-heroism [Warrior]", 450),
    (1, "Potion of Sweet Water", 200),
    (1, "Potion of Treasure Finding", 600),
    (1, "Potion of %s Control",
        _undead_control_potion_table, _typify),
    (1, "Potion of Ventriloquism", 200),
    (1, "Potion of Vitality", 300),
    (1, "Potion of Water Breathing, %s Doses", 
        _water_breathing_potion_table, _typify),
]

def _spell_scroll(row):
    num, n, s, b = row[2]
    name = row[1]
    levs = []
    totlev = 0
    for i in range(num):
        l = Dice.D(n, s, b)
        totlev = totlev + l
        levs.append(str(l))
    levs.sort()
    lstr = string.join(levs, ", ")
    if num > 1:
        name = name + " (Levels " + lstr + ")"
    else:
        name = name + " (Level " + lstr + ")"
    return (0, name, totlev * 100)

_wizard_spell_scroll_table = [
    (3, "Scroll of Wizard Spell",  (1, 1, 4, 0), _spell_scroll),
    (2, "Scroll of Wizard Spell",  (1, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Wizard Spell",  (1, 1, 8, 1), _spell_scroll),
    (1, "Scroll of Wizard Spells", (2, 1, 4, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (2, 1, 8, 1), _spell_scroll),
    (1, "Scroll of Wizard Spells", (3, 1, 4, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (3, 1, 8, 1), _spell_scroll),
    (1, "Scroll of Wizard Spells", (4, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (4, 1, 8, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (5, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (5, 1, 8, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (6, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (6, 1, 6, 2), _spell_scroll),
    (1, "Scroll of Wizard Spells", (7, 1, 8, 0), _spell_scroll),
    (1, "Scroll of Wizard Spells", (7, 1, 8, 1), _spell_scroll),
    (1, "Scroll of Wizard Spells", (7, 1, 6, 3), _spell_scroll),
]

_priest_spell_scroll_table = [
    (3, "Scroll of Priest Spell",  (1, 1, 4, 0), _spell_scroll),
    (2, "Scroll of Priest Spell",  (1, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Priest Spell",  (1, 1, 6, 1), _spell_scroll),
    (1, "Scroll of Priest Spells", (2, 1, 4, 0), _spell_scroll),
    (1, "Scroll of Priest Spells", (2, 1, 6, 1), _spell_scroll),
    (1, "Scroll of Priest Spells", (3, 1, 4, 0), _spell_scroll),
    (1, "Scroll of Priest Spells", (3, 1, 6, 1), _spell_scroll),
    (2, "Scroll of Priest Spells", (4, 1, 6, 0), _spell_scroll),
    (2, "Scroll of Priest Spells", (5, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Priest Spells", (6, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Priest Spells", (6, 1, 4, 2), _spell_scroll),
    (1, "Scroll of Priest Spells", (7, 1, 6, 0), _spell_scroll),
    (1, "Scroll of Priest Spells", (7, 1, 6, 1), _spell_scroll),
    (1, "Scroll of Priest Spells", (7, 1, 4, 3), _spell_scroll),
]

_spell_scroll_table = [
    (7, "Wizard Spell Scroll", _wizard_spell_scroll_table),
    (3, "Priest Spell Scroll", _priest_spell_scroll_table),
]

# personally I think the "all" version is worth more.
_elemental_protection_scroll_table = [
    (15, "Air", 1500),
    (15, "Earth", 1500),
    (15, "Fire", 1500),
    (15, "Water", 1500),
    (10, "All", 2000),
]

# personally I think the "all" version is worth more.
_lycanthrope_protection_scroll_table = [
    ( 5, "Werebears", 1000),
    ( 5, "Wereboars", 1000),
    (10, "Wererats", 1000),
    ( 5, "Weretigers", 1000),
    (15, "Werewolves", 1000),
    (58, "All Lycanthropes", 1500),
    ( 2, "Shape-Changers", 2000),
]

_scroll_table = [
    (38, "Spell Scroll", _spell_scroll_table),
    (1, "Map", 0),
    (1, "Cursed Scroll", 0),
    (1, "Scroll of Protection from Acid", 2500),
    (1, "Scroll of Protection from Cold", 2000),
    (1, "Scroll of Protection from Dragon Breath", 2000),
    (1, "Scroll of Protection from Electricity", 1500),
    (2, "Scroll of Protection from %s Elementals",
        _elemental_protection_scroll_table, _typify),
    (1, "Scroll of Protection from Fire", 2000),
    (1, "Scroll of Protection from Gas", 2000),
    (2, "Scroll of Protection from %s", 
        _lycanthrope_protection_scroll_table, _typify),
    (1, "Scroll of Protection from Magic", 1500),
    (1, "Scroll of Protection from Petrification", 2000),
    (1, "Scroll of Protection from Plants", 1000),
    (1, "Scroll of Protection from Poison", 1000),
    (1, "Scroll of Protection from Possession", 2000),
    (1, "Scroll of Protection from Undead", 1500),
    (1, "Scroll of Protection from Water", 1500),
]

_ring_protection_table = [
    (70, "Ring of Protection +1", 1000),
    (12, "Ring of Protection +2", 2000),
    ( 1, "Ring of Protection +2, 5' Radius", 2000),
    ( 7, "Ring of Protection +3", 3000),
    ( 1, "Ring of Protection +3, 5' Radius", 3000),
    ( 6, "Ring of Protection +4 AC, +2 Save", 4000),
    ( 3, "Ring of Protection +6 AC, +1 Save", 6000),
]

_clumsiness_rings_table = [
    (10, "Free Action", 0),
    (10, "Feather Falling", 0),
    (15, "Invisibility", 0),
    (15, "Jumping", 0),
    (10, "Swimming", 0),
    (20, "Warmth", 0),
    (20, "Water Breathing", 0),
]

_contrariness_rings_table = [
    (20, "Flying", 0),
    (20, "Invisibility", 0),
    (20, "Levitation", 0),
    (10, "Shocking Grasp", 0),
    (10, "Spell Turning", 0),
    (20, "Strength (18/00)", 0),
]

_elemental_command_ring_table = [
    (1, "Air", 5000),
    (1, "Earth", 5000),
    (1, "Fire", 5000),
    (1, "Water", 5000),
]

_regeneration_ring_table = [
    (9, "Ring of Regeneration", 5000),
    (1, "Ring of Vampiric Regeneration", 5000),
]

# I have varied the XP value here.
_telekinesis_ring_table = [
    (25, "25", 1000),
    (25, "50", 1500),
    (39, "100", 2000),
    (10, "200", 2500),
    ( 1, "400", 3000),
]

_wizardry_ring_table = [
    (50, "1st", 4000),
    (25, "2nd", 4000),
    ( 7, "3rd", 4000),
    ( 7, "1st and 2nd", 4000),
    ( 4, "4th", 4000),
    ( 3, "5th", 4000),
    ( 4, "1st, 2nd, and 3rd", 4000),
    ( 1, "4th and 5th", 4000),
]

_rings_table = [
    (2, "Ring of Animal Friendship", 1000),
    (2, "Ring of Blinking", 1000),
    (2, "Ring of Chameleon Power", 1000),
    (2, "Ring of Clumsiness (%s)",
        _clumsiness_rings_table, _typify),
    (2, "Ring of Contrariness (%s)",
        _contrariness_rings_table, _typify),
    (4, "Ring of Delusion", 0),
    (2, "Ring of Djinni Summoning", 3000),
    (2, "Ring of %s Elemental Command", 
        _elemental_command_ring_table, _typify),
    (2, "Ring of Feather Falling", 1000),
    (2, "Ring of Fire Resistance", 1000),
    (2, "Ring of Free Action", 1000),
    (2, "Ring of Human Influence", 2000),
    (2, "Ring of Invisibility", 1500),
    (4, "Ring of Jumping", 1000),
    (2, "Ring of Mammal Control", 1000),
    (2, "Ring of Mind Shielding", 500),
    (4, "Ring of Protection", _ring_protection_table),
    (1, "Ring of the Ram, 6d10 charges", 750),
    (1, "Ring of Regeneration", _regeneration_ring_table),
    (1, "Ring of Shocking Grasp", 1000),
    (1, "Ring of Shooting Stars", 3000),
    (1, "Ring of Spell Storing", 2500),
    (1, "Ring of Spell Turning", 2000),
    (1, "Ring of Sustenance", 500),
    (1, "Ring of Swimming", 1000),
    (1, "Ring of Telekinesis (%s lbs. maximum)", 
        _telekinesis_ring_table, _typify),
    (1, "Ring of Truth", 1000),
    (1, "Ring of Warmth", 1000),
    (1, "Ring of Water Walking", 1000),
    (1, "Ring of Weakness", 0),
    (1, "Ring of Wishes, 2d4", 5000),
    (1, "Ring of Three Wishes", 3000),
    (1, "Ring of Wizardry [Wizard] (doubles %s level spells)",
        _wizardry_ring_table, _typify),
    (1, "Ring of X-Ray Vision", 4000),
]

_rods_table = [
    (2, "Rod of Absorption [Priest, Wizard]", 7500),
    (2, "Rod of Alertness", 7000),
    (1, "Rod of Beguiling [Priest, Wizard, Rogue]", 5000),
    (2, "Rod of Cancellation", 10000),
    (1, "Rod of Flailing", 2000),
    (1, "Rod of Lordly Might [Warrior]", 6000),
    (1, "Rod of Passage", 5000),
    (1, "Rod of Resurrection [Priest]", 10000),
    (1, "Rod of Rulership", 8000),
    (2, "Rod of Security", 3000),
    (2, "Rod of Smiting [Priest, Wizard]", 4000),
    (1, "Rod of Splendor", 2500),
    (2, "Rod of Terror", 3000),
]

_staff_spear_table = [
    (6, "Staff-Spear +1", 1000),
    (4, "Staff-Spear +2", 1500),
    (3, "Staff-Spear +3", 2000),
    (3, "Staff-Spear +4", 2500),
    (3, "Staff-Spear +5", 3000),
    (1, "Staff-Spear +3, as Ranseur", 3500),
]

_staves_table = [
    (2, "Staff-Mace", 1500),
    (1, "Staff of Command [Priest, Wizard]", 5000),
    (2, "Staff of Curing [Priest]", 6000),
    (1, "Staff of Magi [Wizard]", 15000),
    (1, "Staff of Power [Wizard]", 12000),
    (1, "Staff of Serpent [Priest]", 7000),
    (2, "Staff of Slinging [Priest]", 2000),
    (2, "Staff-Spear", _staff_spear_table),
    (2, "Staff of Striking [Priest, Wizard]", 6000),
    (1, "Staff of Swarming Insects [Priest, Wizard]", 5000),
    (1, "Staff of Thunder and Lightning", 8000),
    (2, "Staff of Withering", 8000),
    (1, "Staff of Woodlands [Druid]", 8000),
]

_wands_table = [
    (1, "Wand of Conjuration [Wizard]", 7000),
    (1, "Wand of Earth and Stone", 1000),
    (1, "Wand of Enemy Detection", 2000),
    (1, "Wand of Fear [Priest, Wizard]", 3000),
    (1, "Wand of Fire [Wizard]", 4500),
    (1, "Wand of Flame Extinguishing", 1500),
    (1, "Wand of Frost [Wizard]", 6000),
    (1, "Wand of Illumination", 2000),
    (1, "Wand of Illusion [Wizard]", 3000),
    (1, "Wand of Lightning [Wizard]", 4000),
    (1, "Wand of Magic Detection", 2500),
    (1, "Wand of Magic Missiles", 4000),
    (1, "Wand of Metal and Mineral Detection", 1500),
    (1, "Wand of Negation", 3500),
    (1, "Wand of Paralyzation [Wizard]", 3500),
    (1, "Wand of Polymorphing [Wizard]", 3500),
    (1, "Wand of Secret Door and Trap Location", 5000),
    (1, "Wand of Size Alteration", 3000),
    (1, "Wand of Wonder", 6000),
]

# I fudged XP here
_manual_golems_table = [
    ( 5, ("Clay",  "Priest"), 3000),
    (12, ("Flesh", "Wizard"), 3000),
    ( 1, ("Iron",  "Wizard"), 4000),
    ( 2, ("Stone", "Wizard"), 3500),
]

_misc_tomes_table = [
    (3, "Boccob's Blessed Book [Wizard]", 4500),
    (1, "Book of Exalted Deeds [Priest]", 8000),
    (1, "Book of Infinite Spells", 9000),
    (1, "Book of Vile Darkness [Priest]", 8000),
    (1, "Libram of Gainful Conjuration [Wizard]", 8000),
    (1, "Libram of Ineffable Damnation [Wizard]", 8000),
    (1, "Libram of Silver Magic [Wizard]", 8000),
    (1, "Manual of Bodily Health", 5000),
    (1, "Manual of Gainful Exercise", 5000),
    (1, "Manual of %s Golems [%s]", 
        _manual_golems_table, _typify),
    (1, "Manual of Puissant Skill at Arms [Warrior]", 8000),
    (1, "Manual of Quickness in Action", 5000),
    (1, "Manual of Stealthy Pilfering [Rogue]", 8000),
    (1, "Tome of Clear Thought", 8000),
    (1, "Tome of Leadership and Influence", 7500),
    (1, "Tome of Understanding", 8000),
    (1, "Vacuous Grimoire", 0),
]

_amulet_vs_undead_table = [
    (30, "Amulet Versus Undead, 5th Level", 1000),
    (25, "Amulet Versus Undead, 6th Level", 1200),
    (20, "Amulet Versus Undead, 7th Level", 1400),
    (15, "Amulet Versus Undead, 8th Level", 1600),
    (10, "Amulet Versus Undead, 9th Level", 1800),
]

_pearl_power_table = [
    (25, "Pearl of Power [Wizard], First Level", 200),
    (20, "Pearl of Power [Wizard], Second Level", 400),
    (15, "Pearl of Power [Wizard], Third Level", 600),
    (15, "Pearl of Power [Wizard], Fourth Level", 800),
    (10, "Pearl of Power [Wizard], Fifth Level", 1000),
    ( 7, "Pearl of Power [Wizard], Sixth Level", 1200),
    ( 4, "Pearl of Power [Wizard], Seventh Level", 1400),
    ( 2, "Pearl of Power [Wizard], Eighth Level", 1600),
    ( 1, "Pearl of Power [Wizard], Ninth Level", 1800),
    ( 1, "Pearl of Power [Wizard], Recalls two spells of 1st to 6th Level", 0),
]

_scarab_vs_golems_table = [
    (30, "Scarab Versus Flesh Golems", 400),
    (25, "Scarab Versus Clay Golems", 500),
    (20, "Scarab Versus Stone Golems", 600),
    (10, "Scarab Versus Iron Golems", 800),
    (10, "Scarab Versus Flesh, Clay, and Wood Golems", 900),
    ( 5, "Scarab Versus All Golems", 1250),
]

_necklace_missiles_table = [
    (4, "Necklace of Missiles, 1x5d6, 2x3d6", 2200),
    (4, "Necklace of Missiles, 1x6d6, 2x4d6, 2x2d6", 3600),
    (4, "Necklace of Missiles, 1x7d6, 2x5d6, 4x3d6", 5800),
    (4, "Necklace of Missiles, 1x8d6, 2x6d6, 2x4d6, 4x2d6", 7200),
    (2, "Necklace of Missiles, 1x9d6, 2x7d6, 2x5d6, 2x3d6", 7800),
    (1, "Necklace of Missiles, 1x10d6, 2x8d6, 2x6d6, 4x4d6", 10800),
    (1, "Necklace of Missiles, 1x11d6, 2x9d6, 2x7d6, 2x5d6, 2x3d6", 11800),
]

_medallion_esp_table = [
    (15, "30' Range", 2000),
    ( 3, "30' Range, with Empathy", 2300),
    ( 1, "60' Range", 2500),
    ( 1, "90' Range", 3000),
]

# xp modified here also
_periapt_proof_poison_table = [
    (8, "+1, Special Save 19", 1500),
    (6, "+2, Special Save 17", 1700),
    (8, "+3, Special Save 15", 1500),
    (8, "+4, Special Save 13", 1500),
]

_misc_jewels_table = [
    (1, "Amulet of Inescapable Location", 0),
    (1, "Amulet of Life Protection", 5000),
    (1, "Amulet of the Planes", 6000),
    (1, "Amulet of Proof Against Detection and Location", 4000),
    (1, "Amulet Versus Undead", _amulet_vs_undead_table),
    (1, "Beads of Force", 200),
    (1, "Brooch of Shielding", 1000),
    (1, "Gem of Brightness", 2000),
    (1, "Gem of Insight", 3000),
    (1, "Gem of Seeing", 2000),
    (1, "Jewel of Attacks", 0),
    (1, "Jewel of Flawlessness", 0),
    (1, "Medallion of ESP, %s", _medallion_esp_table, _typify),
    (1, "Medallion of Thought Projection", 0),
    (1, "Necklace of Adaptation", 1000),
    (2, "Necklace of Missiles", _necklace_missiles_table),
    (1, "Necklace of Prayer Beads [Priest] (XP Special)", 0),
    (1, "Necklace of Strangulation", 0),
    (1, "Pearl of Power [Wizard]", _pearl_power_table),
    (1, "Pearl of the Sirines", 900),
    (1, "Pearl of Wisdom [Priest]", 500),
    (1, "Periapt of Foul Rotting", 0),
    (1, "Periapt of Health", 1000),
    (1, "Periapt of Proof Against Poison %s", 
        _periapt_proof_poison_table, _typify),
    (1, "Periapt of Wound Closure", 1000),
    (1, "Phylactery of Faithfulness [Priest]", 1000),
    (1, "Phylactery of Long Years [Priest]", 3000),
    (1, "Phylactery of Monstrous Attention [Priest]", 0),
    (1, "Scarab of Death", 0),
    (1, "Scarab of Enraging Enemies", 1000),
    (1, "Scarab of Insanity", 1500),
    (1, "Scarab of Protection", 2500),
    (1, "Scarab Versus Golems", _scarab_vs_golems_table),
    (1, "Talisman of Pure Good [Priest]", 3500),
    (1, "Talisman of the Sphere [Wizard]", 100),
    (1, "Talisman of Ultimate Evil [Priest]", 3500),
    (1, "Talisman of Zagy", 1000),
]

_cloak_protection_table = [
    (35, "Cloak of Protection +1", 1000),
    (30, "Cloak of Protection +2", 2000),
    (20, "Cloak of Protection +3", 3000),
    (10, "Cloak of Protection +4", 4000),
    ( 5, "Cloak of Protection +5", 5000),
]

_misc_cloaks_table = [
    (1, "Cloak of Arachnida", 3000),
    (1, "Cloak of Displacement", 3000),
    (2, "Cloak of Elvenkind", 1000),
    (1, "Cloak of Poisonousness", 0),
    (2, "Cloak of Protection", _cloak_protection_table),
    (1, "Cloak of the Bat", 1500),
    (1, "Cloak of the Manta Ray", 2000),
    (1, "Robe of the Archmagi [Wizard]", 6000),
    (1, "Robe of Blending", 3500),
    (1, "Robe of Eyes [Wizard]", 4500),
    (1, "Robe of Powerlessness [Wizard]", 0),
    (1, "Robe of Scintillating Colors [Priest, Wizard]", 2750),
    (1, "Robe of Stars [Wizard]", 4000),
    (2, "Robe of Useful Items [Wizard]", 1500),
    (1, "Robe of Vermin [Wizard]", 0),
]

# I adjusted this table because the odds seemed wrong.
_bracers_defense_table = [
    ( 5, "Bracers of Defense, AC 8", 1000),
    (10, "Bracers of Defense, AC 7", 1500),
    (15, "Bracers of Defense, AC 6", 2000),
    (20, "Bracers of Defense, AC 5", 2500),
    (20, "Bracers of Defense, AC 4", 3000),
    (15, "Bracers of Defense, AC 3", 3500),
    (15, "Bracers of Defense, AC 2", 4000),
]

def _generate_boots_varied_tracks(row):
    tbl = [ "Basilisk", "Bear", "Boar", "Bull", 
            "Camel", "Dog", "Hill Giant", "Goat", 
            "Horse", "Lion", "Mule", "Rabbit", 
            "Stag", "Tiger", "Wolf", "Wyvern" ]
    which = []
    for i in range(4):
        n = Dice.D(1, len(tbl), -1)
        which.append(tbl[n])
        del tbl[n]
    return (0, row[1] + " (" + string.join(which, ", ") + ")", 1500)

_winged_boots_table = [
    (1, '15" Movement, MC A', 2000),
    (1, '18" Movement, MC B', 2000),
    (1, '21" Movement, MC C', 2000),
    (1, '24" Movement, MC D', 2000),
]

_misc_boots_table = [
    (1, "Boots of Dancing", 0),
    (1, "Boots of Elvenkind", 1000),
    (1, "Boots of Levitation", 2000),
    (1, "Boots of Speed", 2500),
    (1, "Boots of Striding and Springing", 2500),
    (1, "Boots of the North", 1500),
    (1, "Boots of Varied Tracks", _generate_boots_varied_tracks),
    (1, "Boots, Winged (%s)", 
        _winged_boots_table, _typify),
    (1, "Bracers of Archery [Warrior]", 1000),
    (1, "Bracers of Brachiation", 1000),
    (2, "Bracers of Defense", _bracers_defense_table),
    (1, "Bracers of Defenselessness", 0),
    (1, "Gauntlets of Dexterity", 1000),
    (1, "Gauntets of Fumbling", 0),
    (1, "Gauntlets of Ogre Power [Priest, Rogue, Warrior]", 1000),
    (1, "Gauntlets of Swimming and Climbing [Priest, Rogue, Warrior]", 1000),
    (1, "Gloves of Missile Snaring", 1500),
    (1, "Slippers of Spider Climbing", 1000),
]
# 500 * Per AC of protection less than 10

# I modified the XP here.
_girdle_giant_strength_table = [
    (30, "Hill",  1500),
    (20, "Stone", 1800),
    (20, "Frost", 2000),
    (15, "Fire",  2300),
    (10, "Cloud", 2600),
    ( 5, "Storm", 3000),
]

_misc_girdles_table = [
    (3, "Girdle of Dwarvenkind", 3500),
    (1, "Girdle of Femininity/Masculinity [Priest, Rogue, Warrior]", 0),
    (2, "Girdle of %s Giant Strength [Priest, Rogue, Warrior]", 
        _girdle_giant_strength_table, _typify),
    (3, "Girdle of Many Pouches", 1000),
    (1, "Hat of Disguise", 1000),
    (1, "Hat of Stupidity", 0),
    (1, "Helm of Brilliance", 2500),
    (2, "Helm of Comprehending Languages and Reading Magic", 1000),
    (1, "Helm of Opposite Alignment", 0),
    (1, "Helm of Telepathy", 3000),
    (1, "Helm of Teleportation", 2500),
    (2, "Helm of Underwater Action", 1000),
]

# XP values of these items are missing in the rules, so I guessed.
_bucknards_purse_table = [
    (5, "Bucknard's Everfull Purse (26 SP, 26 EP, 26 GP)", 500),
    (4, "Bucknard's Everfull Purse (26 CP, 26 EP, 26 PP)", 500),
    (1, "Bucknard's Everfull Purse (26 CP, 26 EP, 26 Gems)", 1000),
]

_bag_holding_capacity_table = [
    (3, "Up to 250 lbs/30 cu. ft weighs 15 lbs.", 4000),
    (4, "Up to 500 lbs/70 cu. ft weighs 15 lbs.", 5000),
    (2, "Up to 1000 lbs/150 cu. ft weighs 35 lbs.", 5500),
    (1, "Up to 1500 lbs/250 cu. ft weighs 60 lbs.", 6000),
]

# does anyone know why this is a 0 XP item?
_iron_flask_table = [
    (50, "Empty", 0),
    ( 4, "Air Elemental", 0),
    (10, "Djinni", 0),
    ( 4, "Earth Elemental", 0),
    ( 3, "Efreeti", 0),
    ( 4, "Fire Elemental", 0),
    (10, "Invisible Stalker", 0),
    ( 3, "Rakshasa", 0),
    ( 4, "Salamander", 0),
    ( 4, "Water Elemental", 0),
    ( 2, "Wind Walker", 0),
    ( 1, "Xorn", 0),
]

_misc_bags_table = [
    (1, "Alchemy Jug", 3000),
    (1, "Bag of Beans", 1000),
    (1, "Bag of Devouring", 0),
    (4, "Bag of Holding (%s)", 
        _bag_holding_capacity_table, _typify),
    (1, "Bag of Transmuting", 0),
    (1, "Bag of Tricks", 2500),
    (1, "Beaker of Plentiful Potions", 1500),
    (1, "Bucknard's Everfull Purse", _bucknards_purse_table),
    (1, "Decanter of Endless Water", 1000),
    (1, "Efreeti Bottle", 9000),
    (1, "Eversmoking Bottle", 500),
    (1, "Flask of Curses", 0),
    (1, "Heward's Handy Haversack", 3000),
    (1, "Iron Flask (%s)", _iron_flask_table, _typify),
    (1, "Portable Hole", 5000),
    (1, "Pouch of Accessibility", 1500),
]

def _quantify(row):
    name = row[1]
    n, s, b, mul, xp = row[2]
    num = Dice.D(n, s, b) * mul
    xp = xp * num
    name = name + (" (%d)" % num)
    return (0, name, xp)

def _ioun_stones(row):
    oplst = [
        'Pale Blue Rhomboid, +1 Strength',
        'Scarlet and Blue Sphere, +1 Intelligence',
        'Incandescent Blue Sphere, +1 Wisdom',
        'Deep Red Sphere, +1 Dexterity',
        'Pink Rhomboid, +1 Constitution',
        'Pink and Green Sphere, +1 Charisma',
        'Pale Green Prism, +1 Level',
        'Clear Spindle, Sustains User Without Food/Water',
        'Iridescent Spindle, Sustains User Without Air',
        'Pearly White Spindle, Regenerates 1 HP/Turn',
        'Pale Lavender Ellipsoid, Absorbs up to 4th Level Spells',
        'Lavender and Green Ellipsoid, Absorbs up to 8th Level Spells',
        'Vibrant Purple Prism, Stores 2d6 Spell Levels',
        'Dusty Rose Prism, +1 Protection',
        'Dull Gray, Any Shape, Burned Out',
        'Dull Gray, Any Shape, Burned Out',
        'Dull Gray, Any Shape, Burned Out',
        'Dull Gray, Any Shape, Burned Out',
        'Dull Gray, Any Shape, Burned Out',
        'Dull Gray, Any Shape, Burned Out',
    ]
    num = Dice.D(1, 10, 0)
    stones = []
    for i in range(num):
        sel = Dice.D(1, len(oplst), -1)
        stones.append(oplst[sel])
        del oplst[sel]
    stones.sort()
    return (0, "%d Ioun Stones" % num, 300 * num, stones)

_misc_candles_table = [
    (1, "Candle of Invocation [Priest]", 1000),
    (1, "Dust of Appearance", 1000),
    (1, "Dust of Disappearance", 2000),
    (1, "Dust of Dryness", 1000),
    (1, "Dust of Illusion", 1000),
    (1, "Dust of Tracelessness", 500),
    (1, "Dust of Sneezing and Choking", 0),
    (1, "Incense of Meditation [Priest]", 500),
    (1, "Incense of Obsession [Priest]", 0),
    (1, "Ioun Stones", _ioun_stones),
        #(1, 10, 0, 1, 300), _quantify),
    (1, "Keoghtom's Ointment", 500),
    (1, "Nolzur's Marvelous Pigments", (1, 4, 0, 1, 500), _quantify),
    (1, "Philosopher's Stone", 1000),
    (1, "Smoke Powder", 0),
    (1, "Sovereign Glue", 1000),
    (1, "Stone of Controlling Earth Elementals", 1500),
    (1, "Stone of Good Luck (Luckstone)", 3000),
    (1, "Stone of Weight (Loadstone)", 0),
    (1, "Universal Solvent", 1000),
]

_carpet_flying_table = [
    (20, "3'x5', 1 Seat, 42\" Speed", 7500),
    (35, "4'x6', 2 Seat, 36\" Speed", 7500),
    (20, "5'x7', 3 Seat, 30\" Speed", 7500),
    (20, "6'x9', 4 Seat, 24\" Speed", 7500),
]

_misc_tools_table = [
    (1, "Brazier Commanding Fire Elementals [Wizard]", 4000),
    (1, "Brazier of Sleep Smoke [Wizard]", 0),
    (1, "Broom of Animated Attack", 0),
    (1, "Broom of Flying", 2000),
    (1, "Carpet of Flying (%s)", 
        _carpet_flying_table, _typify),
    (1, "Mattock of the Titans [Warrior]", 3500),
    (1, "Maul of the Titans [Warrior]", 4000),
    (1, "Mirror of Life Trapping [Wizard]", 2500),
    (1, "Mirror of Mental Prowess", 5000),
    (1, "Mirror of Opposition", 0),
    (1, "Murlynd's Spoon", 750),
    (2, "Rope of Climbing", 1000),
    (1, "Rope of Constriction", 0),
    (1, "Rope of Entanglement", 1500),
    (1, "Rug of Smothering", 0),
    (1, "Rug of Welcome [Wizard]", 6500),
    (1, "Saw of Mighty Cutting [Warrior]", 2000),
    (1, "Spade of Colossal Excavation [Warrior]", 1000),
]

# here also I have modified the XP
_horn_valhalla_table = [
    (8, "Silver",  750),
    (7, "Brass",  1000),
    (3, "Bronze", 1250),
    (2, "Iron",   1500),
]

_misc_music_table = [
    (1, "Chime of Interruption", 2000),
    (1, "Chime of Opening", 3500),
    (1, "Chime of Hunger", 0),
    (1, "Drums of Deafening", 0),
    (1, "Drums of Panic", 6500),
    (1, "Harp of Charming", 5000),
    (1, "Harp of Discord", 0),
    (1, "Horn of Blasting", 1000),
    (1, "Horn of Bubbles", 0),
    (1, "Horn of Collapsing", 1500),
    (1, "Horn of Fog", 400),
    (1, "Horn of Goodness (Evil)", 750),
    (1, "Horn of the Tritons [Priest, Warrior]", 2000),
    (1, "%s Horn of Valhalla", 
        _horn_valhalla_table, _typify),
    (1, "Lyre of Building", 5000),
    (1, "Pipes of Haunting", 400),
    (1, "Pipes of Pain", 0),
    (1, "Pipes of Sounding", 1000),
    (1, "Pipes of the Sewers", 2000),
]

_figurine_elephant_table = [
    (9, "", 1000),
    (1, ", Prehistoric", 1000),
]

_figurine_table = [
    (15, "Figurine of Wondrous Power, Ebony Fly", 400),
    (15, "Figurine of Wondrous Power, Golden Lions (pair)", 1000),
    # I fudged on the XP for the goats
    (10, "Figurine of Wondrous Power, Ivory Goats (trio)", 2000),
    (15, "Figurine of Wondrous Power, Marble Elephant%s",
        _figurine_elephant_table, _typify),
    (10, "Figurine of Wondrous Power, Obsidian Steed", 400),
    # TODO: Check HD and correct XP
    (20, "Figurine of Wondrous Power, Onyx Dog", 300),
    (15, "Figurine of Wondrous Power, Serpentine Owl", 200),
]

_quaal_feather_token_table = [
    (4, "Anchor", 1000),
    (3, "Bird", 1000),
    (3, "Fan", 1000),
    (3, "Swan Boat", 1000),
    (5, "Tree", 1000),
    (2, "Whip", 1000),
]

_misc_weird_table = [
    (1, "Apparatus of Kwalish", 8000),
    (2, "Boat, Folding", 10000),
    (1, "Bowl Commanding Water Elementals [Wizard]", 4000),
    (1, "Bowl of Watery Death [Wizard]", 0),
    (1, "Censer Controlling Air Elementals [Wizard]", 4000),
    (1, "Censer of Summoning Hostile Air Elementals [Wizard]", 0),
    (2, "Crystal Ball [Wizard]", 1000),
    (1, "Crystal Hypnosis Ball [Wizard]", 0),
    (1, "Cube of Force", 3000),
    (2, "Cube of Frost Resistance", 2000),
    (1, "Cubic Gate", 5000),
    (1, "Daern's Instant Fortress", 7000),
    (1, "Deck of Illusions", 1500),
    (1, "Deck of Many Things", 0),
    (1, "Eyes of Charming [Wizard]", 4000),
    (1, "Eyes of Minute Seeing", 2000),
    (1, "Eyes of Petrification", 0),
    (1, "Eyes of the Eagle", 3500),
    (2, "Figurine of Wondrous Power", _figurine_table),
    (1, "Horseshoes of a Zephyr", 1500),
    (2, "Horseshoes of Speed", 2000),
    (1, "Iron Bands of Bilarro", 750),
    (1, "Lens of Detection", 250),
    (1, "Quaal's Feather Token (%s)", 
        _quaal_feather_token_table, _typify),
    (2, "Quiver of Ehlonna", 1500),
    (1, "Sheet of Smallness", 1500),
    (1, "Sphere of Annihilation", 4000),
    (1, "Stone Horse", 2000),
    (1, "Well of Many Worlds", 6000),
    (2, "Wind Fan", 500),
    (1, "Wings of Flying", 750),
]

__sword_adjustment_table = [
    (2, "-1",    0),
    (8, "+1",  400),
    (4, "+2",  800),
    (3, "+3", 1400),
    (2, "+4", 2000),
    (1, "+5", 3000),
]

__other_adjustment_table = [
    (2, "-1",    0),
    (8, "+1",  500),
    (4, "+1",  500),
    (3, "+2", 1000),
    (2, "+2", 1000),
    (1, "+3", 2000),
]

__sword_type_table = [
    (70, "Longsword"),
    (20, "Scimitar"),
    ( 5, "Shortsword"),
    ( 4, "Bastardsword"),
    ( 1, "Greatsword"),
]

def _generate_weapon(row):
    addl = Dice.Roll(__other_adjustment_table)
    return (0, row[1] + " " + addl[1], addl[2])

def _typify_sword(row):
    name = row[1]
    if name[:5] == "Sword":
        typ = Dice.Roll(__sword_type_table)
        name = typ[1] + name[5:]
    return (0, name, row[2])

def _generate_sword(row):
    addl = Dice.Roll(__sword_adjustment_table)
    return _typify_sword((0, row[1] + " " + addl[1], addl[2]))

def _generate_ammo(row):
    dr = row[2]
    qty = Dice.D(dr[0], dr[1], dr[2])
    name = str(qty) + " " + row[1]
    addl = Dice.Roll(__other_adjustment_table)
    return (0, name + " " + addl[1], addl[2] * qty)

_axe_hurling_table = [
    (5, "Axe of Hurling +1", 1500),
    (5, "Axe of Hurling +2", 3000),
    (5, "Axe of Hurling +3", 4500),
    (4, "Axe of Hurling +4", 6000),
    (1, "Axe of Hurling +5", 7500),
]

_dagger_throwing_table = [
    (35, "Dagger of Throwing +1", 250),
    (30, "Dagger of Throwing +2", 350),
    (25, "Dagger of Throwing +3", 450),
    (10, "Dagger of Throwing +4", 550),
]

# I pulled the odds for this table out of thin air.
_hornblade_table = [
    (3, "Hornblade +1, Knife-Size",     500),
    (2, "Hornblade +2, Knife-Size",    1000),
    (1, "Hornblade +3, Knife-Size",    1000),
    (2, "Hornblade +1, Dagger-Size",    750),
    (3, "Hornblade +2, Dagger-Size",   1500),
    (1, "Hornblade +3, Dagger-Size",   1500),
    (1, "Hornblade +1, Scimitar-Size", 1000),
    (3, "Hornblade +2, Scimitar-Size", 2000),
    (2, "Hornblade +3, Scimitar-Size", 3000),
]

_scimitar_speed_table = [
    (10, "Scimitar of Speed +1", 2500),
    (60, "Scimitar of Speed +2", 3000),
    ( 5, "Scimitar of Speed +3", 3500),
    ( 3, "Scimitar of Speed +4", 4000),
    ( 2, "Scimitar of Speed +5", 4500),
]

_quarterstaff_table = [
    (5, "Quarterstaff, Magical, +1", 250),
    (4, "Quarterstaff, Magical, +2", 500),
    (4, "Quarterstaff, Magical, +3", 750),
    (4, "Quarterstaff, Magical, +4", 1000),
    (3, "Quarterstaff, Magical, +5", 1250),
]

_arrow_slaying_table = [
    (1, "Arachnids", 250),
    (1, "Avians", 250),
    (1, "Bards", 250),
    (1, "Clerics", 250),
    (1, "Dragons", 250),
    (1, "Druids", 250),
    (1, "Elementals", 250),
    (1, "Fighters", 250),
    (1, "Giants", 250),
    (1, "Golems", 250),
    (1, "Illusionists", 250),
    (1, "Mages", 250),
    (1, "Mammals", 250),
    (1, "Paladins", 250),
    (1, "Rangers", 250),
    (1, "Reptiles", 250),
    (1, "Sea Monsters", 250),
    (1, "Thieves", 250),
    (1, "Titans", 250),
    (1, "Undead", 250),
]

__dragon_slayer_table = [
    (1, "Black"),
    (1, "Blue"),
    (1, "Brass"),
    (1, "Bronze"),
    (1, "Copper"),
    (1, "Gold"),
    (1, "Green"),
    (1, "Red"),
    (1, "Silver"),
    (1, "White"),
]

def _generate_dragon_slayer(row):
    addl = Dice.Roll(__dragon_slayer_table)
    name = row[1] % addl[1]
    return _typify_sword((0, name, 900))

_special_weapon_table = [
    ( 3, "Arrow of Direction", 2500),
    ( 3, "Arrow of Slaying %s", 
        _arrow_slaying_table, _typify),
    ( 3, "Axe +2, Throwing", 750),
    ( 3, "Axe of Hurling", _axe_hurling_table),
    ( 6, "Bow +1", 500),
    ( 3, "Crossbow of Accuracy, +3", 2000),
    ( 3, "Crossbow of Distance", 1500),
    ( 3, "Crossbow of Speed", 1500),
    ( 6, "Dagger +1, +2 vs. Tiny or Small creatures", 300),
    ( 6, "Dagger +2, +3 vs. larger than man-sized", 300),
    ( 3, "Dagger +2, Longtooth", 300),
    ( 3, "Dagger of Throwing", _dagger_throwing_table),
    ( 3, "Dagger of Venom", 350),
    ( 3, "Dart of Homing", 450),
    ( 3, "Hammer +3, Dwarven Thrower", 1500),
    ( 3, "Hammer of Thunderbolts", 2500),
    ( 3, "Hornblade", _hornblade_table),
    ( 3, "Javelin of Lightning", 250),
    ( 3, "Javelin of Piercing", 250),
    ( 6, "Knife, Buckle", 150),
    ( 6, "Mace of Disruption", 2000),
    ( 3, "Net of Entrapment", 1000),
    ( 3, "Net of Snaring", 1000),
    ( 6, "Quarterstaff, Magical", _quarterstaff_table),
    ( 3, "Scimitar of Speed", _scimitar_speed_table),
    ( 6, "Sling of Seeking +2", 700),
    ( 3, "Spear, Cursed Backbiter", 0),
    ( 3, "Trident of Fish Command", 500),
    ( 3, "Trident of Submission", 1500),
    ( 3, "Trident of Warning", 1000),
    ( 3, "Trident of Yearning", 0),
    ( 3, "Sun Blade", 3000),
    (18, "Sword +1, +2 vs. magic-using and enchanted creatures", 
        600, _typify_sword),
    ( 9, "Sword +1, +3 vs. lycanthropes and shape-changers", 
        700, _typify_sword),
    ( 6, "Sword +1, +3 vs. regenerating creatures", 800, _typify_sword),
    ( 3, "Sword +1, +4 vs. reptiles", 800, _typify_sword),
    ( 6, "Sword +1, Cursed", 0, _typify_sword),
    ( 3, "Sword +1, Flame Tongue", 900, _typify_sword),
    ( 3, "Sword +1, Luck Blade", 1000, _typify_sword),
    ( 3, "Sword +2, %s Dragon Slayer", _generate_dragon_slayer),
    ( 3, "Sword +2, Giant Slayer", 900, _typify_sword),
    ( 1, "Sword +2, Nine Lives Stealer", 1600, _typify_sword),
    ( 2, "Sword +3, Frost Brand", 1600, _typify_sword),
    ( 1, "Sword +4, Defender", 3000, _typify_sword),
    ( 1, "Sword +5, Defender", 3600, _typify_sword),
    ( 1, "Sword +5, Holy Avenger", 4000, _typify_sword),
    ( 2, "Sword -2, Cursed", 0, _typify_sword),
    ( 1, "Sword of Dancing", 4400, _typify_sword),
    ( 1, "Sword of Life Stealing", 5000, _typify_sword),
    ( 1, "Sword of Sharpness", 7000, _typify_sword),
    ( 1, "Sword of the Planes", 2000, _typify_sword),
    ( 1, "Sword of Wounding", 4400, _typify_sword),
    ( 3, "Sword, Cursed Berserking", 0, _typify_sword),
    ( 2, "Sword, Short, Quickness (+2)", 1000, _typify_sword),
    ( 1, "Sword, Vorpal Weapon", 10000, _typify_sword),
]

_weapon_table = [
    ( 1, "Arrows", (4, 6, 0), _generate_ammo),
    ( 1, "Arrows", (3, 6, 0), _generate_ammo),
    ( 1, "Arrows", (2, 6, 0), _generate_ammo),
    ( 2, "Axe", _generate_weapon),
    ( 1, "Battle Axe", _generate_weapon),
    ( 1, "Bolt", (2, 10, 0), _generate_ammo),
    ( 1, "Bolt", (2, 6, 0), _generate_ammo),
    ( 1, "Sling Bullet", (3, 4, 0), _generate_ammo),
    ( 3, "Dagger", _generate_weapon),
    ( 1, "Darts", (3, 4, 0), _generate_ammo),
    ( 1, "Flail", _generate_weapon),
    ( 1, "Javelin(s)", (1, 2, 0), _generate_ammo),
    ( 1, "Knife", _generate_weapon),
    ( 1, "Lance", _generate_weapon),
    ( 2, "Mace", _generate_weapon),
    ( 2, "Military Pick", _generate_weapon),
    ( 2, "Morning Star", _generate_weapon),
    ( 2, "Pole Arm", _generate_weapon),
    ( 4, "Scimitar", _generate_weapon),
    ( 6, "Spear", _generate_weapon),
    (18, "Sword", _generate_sword),
    ( 2, "Trident", _generate_weapon),
    ( 2, "Warhammer", _generate_weapon),
    ( 3, "Special", _special_weapon_table),
]

__armor_adjustment_table = [
    (2, "-1", 0),
    (8, "+1", 500),
    (4, "+2", 1000),
    (3, "+3", 1500),
    (2, "+4", 2000),
    (1, "+5", 3000),
]

def _generate_armor(row):
    addl = Dice.Roll(__armor_adjustment_table)
    return (0, row[1] + " " + addl[1], addl[2])

def _generate_special_armor(row):
    addl = Dice.Roll(__armor_adjustment_table)
    return (0, row[1] + " " + addl[1], addl[2] + row[2])

## I allow armor of command and armor of rage to be other than +1
_special_armor_table = [
    (1, "Armor of Command", 1000, _generate_special_armor),
    (1, "Armor of Blending", 500, _generate_special_armor),
    (1, "Armor of Missile Attraction", 0),
    (1, "Armor of Rage", 0),
# TODO: need to generate sizes also
    (1, "Elven Chain Mail", 1000, _generate_special_armor),
    (1, "Plate Mail of Etherealness", 5000),
    (1, "Plate Mail of Fear", 4000),
    (1, "Plate Mail of Vulnerability", 0),
    (1, "Shield, Large, +1, +4 vs. Missiles", 400),
    (1, "Shield -1, Missile Attractor", 0),
]

_armor_table = [
    (1, "Banded Mail", _generate_armor),
    (1, "Brigandine", _generate_armor),
    (3, "Chain Mail", _generate_armor),
    (1, "Field Plate", _generate_armor),
    (1, "Full Plate", _generate_armor),
    (1, "Leather", _generate_armor),
    (4, "Plate Mail", _generate_armor),
    (1, "Ring Mail", _generate_armor),
    (1, "Scale Mail", _generate_armor),
    (3, "Shield", _generate_armor),
    (1, "Splint Mail", _generate_armor),
    (1, "Studded Leather", _generate_armor),
    (1, "Special", _special_armor_table),
]

_misc_table = [
    ( 1, "Misc: Books and Tomes", _misc_tomes_table),
    ( 2, "Misc: Jewels and Jewelry", _misc_jewels_table),
    ( 2, "Misc: Cloaks and Robes", _misc_cloaks_table),
    ( 2, "Misc: Boots and Gloves", _misc_boots_table),
    ( 1, "Misc: Girdles and Helms", _misc_girdles_table),
    ( 2, "Misc: Bags and Bottles", _misc_bags_table),
    ( 1, "Misc: Dusts and Stones", _misc_candles_table),
    ( 1, "Misc: Household Items and Tools", _misc_tools_table),
    ( 1, "Misc: Musical Instruments", _misc_music_table),
    ( 2, "Misc: The Weird Stuff", _misc_weird_table),
]

_wpn_armor_table = [
    (15, "Armor and Shields", _armor_table),
    (25, "Weapons", _weapon_table),
]

_gen_magic_table = [
    (20, "Potions and Oils", _potion_table),
    (15, "Scrolls", _scroll_table),
    ( 5, "Rings", _rings_table),
    ( 1, "Rods", _rods_table),
    ( 1, "Staves", _staves_table),
    ( 3, "Wands", _wands_table),
]

_magic_table = [
    (45, "General Magic", _gen_magic_table),
    (15, "Miscellaneous Magic", _misc_table),
    (40, "Armor and Weapons", _wpn_armor_table),
]

_non_weapon_item_table = [
    (45, "General Magic", _gen_magic_table),
    (15, "Miscellaneous Magic", _misc_table),
]

class Magic(_Treasure.Item):
    __magic_switch = {
        "AN": _magic_table,
        "PO": _potion_table,
        "SC": _scroll_table,
        "AW": _wpn_armor_table,
        "NO": _non_weapon_item_table,
        "RI": _rings_table,
        "RO": _rods_table,
        "ST": _staves_table,
        "WA": _wands_table,
        "MI": _misc_table,
        "AR": _armor_table,
        "WE": _weapon_table,
    }
    def __init__(self, kind = "Any"):
        _Treasure.Item.__init__(self)
        self.cat = "Magic"
        self.fullcat = self.fullcat + "." + self.cat
        row = Dice.MRoll(self.__magic_switch[string.upper(kind)[:2]])
        self.name = row[1]
        self.xp = row[2]
        if len(row) > 3:
            self.desc = row[3]
    def __str__(self):
        s = self.cat + ": "
        if self.qty != 1:
            s = s + " " + str(self.qty)
        s = s + " " + self.name
        s = s + " " + str(self.xp) + " XP"
        if self.qty != 1:
            s = s + " ea. (" \
                + str(float(self.xp) * self.qty) + " XP total)"
        if self.desc:
            s = s + (" [%d sub-items]" % len(self.desc))
        return s

###############################################################################
#  Namespace cleanup
###############################################################################

del _magic_table
del _potion_table
del _spell_scroll_table
del _scroll_table
del _ring_protection_table
del _rings_table
del _rods_table
del _staff_spear_table
del _staves_table
del _wands_table
del _misc_tomes_table
del _amulet_vs_undead_table
del _pearl_power_table
del _scarab_vs_golems_table
del _necklace_missiles_table
del _misc_jewels_table
del _cloak_protection_table
del _misc_cloaks_table
del _axe_hurling_table
del _dagger_throwing_table
del _hornblade_table
del _scimitar_speed_table
del _special_weapon_table
del _weapon_table
del _wpn_armor_table
del _armor_table
del _special_armor_table
del _non_weapon_item_table
del _generate_armor
del _generate_special_armor
del _generate_weapon
del _generate_sword
del _generate_ammo

if __name__ == '__main__':
    import sys

    if len(sys.argv) > 1:
        # args should be [ [ n ] t ]
        if len(sys.argv) == 2:
            n = 1
            t = sys.argv[1]
        else:
            n = int(sys.argv[1])
            t = sys.argv[2]
        for i in range(n):
            print Magic(t)
    else:
        print Magic()

# end of file.
