#!/usr/bin/env python
###############################################################################
#  Treasure.py -- generate AD&D treasures
###############################################################################

import Gems, Art, Coins, Magic
import Dice
import string

def _gen_coins(argtup):
    kind, n, s, b, mul = argtup
    return [ Coins.Coin(kind, (Dice.D(n, s, b) * mul)) ]

def _gen_gems(argtup):
    n, s, b, mul = argtup
    lst = []
    qty = Dice.D(n, s, b) * mul
    for i in range(qty):
        lst = lst + [ Gems.Gem() ]
    return lst

def _gen_art(argtup):
    n, s, b, mul = argtup
    lst = []
    qty = Dice.D(n, s, b) * mul
    for i in range(qty):
        lst = lst + [ Art.Art() ]
    return lst

def __gen_magic(argtup):
    kind, n, s, b, mul = argtup
    lst = []
    qty = Dice.D(n, s, b) * mul
    for i in range(qty):
        lst = lst + [ Magic.Magic(kind) ]
    return lst

def _gen_magic(argtup):
    if type(argtup) is type([]):
        lst = []
        for i in argtup:
            lst = lst + __gen_magic(i)
        return lst
    else:
        return __gen_magic(argtup)

_treasure_table = {

    # lair treasure

    'A': [
            (25, _gen_coins, ("CP", 10,  3, 0, 100)),
            (30, _gen_coins, ("SP",  2, 10, 0, 100)),
            (40, _gen_coins, ("GP", 10,  6, 0, 100)),
            (35, _gen_coins, ("PP",  3,  6, 0, 100)),
            (60, _gen_gems,  (1, 31, 9, 1)),
            (50, _gen_art,   (2, 6, 0, 1)),
            (30, _gen_magic, ("Any", 0, 0, 3, 1)),
         ],
    'B': [
            (50, _gen_coins, ("CP", 10,  6, 0,  100)),
            (25, _gen_coins, ("SP", 10,  3, 0,  100)),
            (25, _gen_coins, ("GP",  2, 10, 0,  100)),
            (25, _gen_coins, ("PP",  1, 10, 0,  100)),
            (30, _gen_gems,  (1, 8, 0, 1)),
            (20, _gen_art,   (1, 4, 0, 1)),
            (10, _gen_magic, ("AW", 0, 0, 1, 1)),
         ],
    'C': [
            (20, _gen_coins, ("CP", 10, 10, 0,  100)),
            (30, _gen_coins, ("SP", 10,  6, 0,  100)),
            (10, _gen_coins, ("EP",  1,  6, 0,  100)),
            (25, _gen_gems,  (1, 6, 0, 1)),
            (20, _gen_art,   (1, 3, 0, 1)),
            (10, _gen_magic, ("Any", 0, 0, 2, 1)),
         ],
    'D': [
            (10, _gen_coins, ("CP", 10,  6, 0,  100)),
            (15, _gen_coins, ("SP", 10, 10, 0,  100)),
            (50, _gen_coins, ("GP", 10,  3, 0,  100)),
            (15, _gen_coins, ("PP",  1,  6, 0,  100)),
            (30, _gen_gems,  (1, 10, 0, 1)),
            (25, _gen_art,   (1,  6, 0, 1)),
            (15, _gen_magic, [
                ("Any",    0, 0, 2, 1),
                ("Potion", 0, 0, 1, 1),
            ]),
         ],
    'E': [
            ( 5, _gen_coins, ("CP", 10,  6, 0,  100)),
            (25, _gen_coins, ("SP", 10, 10, 0,  100)),
            (25, _gen_coins, ("GP", 10,  4, 0,  100)),
            (25, _gen_coins, ("PP",  3,  6, 0,  100)),
            (15, _gen_gems,  (1, 12, 0, 1)),
            (10, _gen_art,   (1,  6, 0, 1)),
            (25, _gen_magic, [
                ("Any",    0, 0, 3, 1),
                ("Scroll", 0, 0, 1, 1),
            ]),
         ],
    'F': [
            (10, _gen_coins, ("SP",  3,  6, 0, 1000)),
            (40, _gen_coins, ("GP", 10,  6, 0,  100)),
            (15, _gen_coins, ("PP", 10,  4, 0,  100)),
            (20, _gen_gems,  (2, 10, 0, 1)),
            (10, _gen_art,   (1,  8, 0, 1)),
            (30, _gen_magic, ("Non-Weapon", 0, 0, 5, 1)),
         ],
    'G': [
            (50, _gen_coins, ("GP",  2, 10, 0, 1000)),
            (50, _gen_coins, ("PP",  1, 10, 0, 1000)),
            (30, _gen_gems,  (3,  6, 0, 1)),
            (25, _gen_art,   (1,  6, 0, 1)),
            (35, _gen_magic, ("Any", 0, 0, 5, 1)),
         ],
    'H': [
            (25, _gen_coins, ("CP",  3,  6, 0, 1000)),
            (40, _gen_coins, ("SP",  2, 10, 0, 1000)),
            (55, _gen_coins, ("GP",  2, 10, 0, 1000)),
            (40, _gen_coins, ("PP",  1,  8, 0, 1000)),
            (50, _gen_gems,  (3, 10, 0, 1)),
            (50, _gen_art,   (2, 10, 0, 1)),
            (15, _gen_magic, ("Any", 0, 0, 6, 1)),
         ],
    'I': [
            (30, _gen_coins, ("PP",  1,  6, 0, 100)),
            (55, _gen_gems,  (2, 6, 0, 1)),
            (50, _gen_art,   (2, 4, 0, 1)),
            (15, _gen_magic, ("Any", 0, 0, 1, 1)),
         ],

    # personal treasure

    'J': [
            (100, _gen_coins, ("CP", 3,  8, 0, 1)),
         ],
    'K': [
            (100, _gen_coins, ("SP", 3,  6, 0, 1)),
         ],
    'L': [
            (100, _gen_coins, ("PP", 2,  6, 0, 1)),
         ],
    'M': [
            (100, _gen_coins, ("GP", 2,  4, 0, 1)),
         ],
    'N': [
            (100, _gen_coins, ("PP", 1,  6, 0, 1)),
         ],
    'O': [
            (100, _gen_coins, ("CP", 10,  4, 0, 1)),
            (100, _gen_coins, ("SP", 10,  3, 0, 1)),
         ],
    'P': [
            (100, _gen_coins, ("SP", 10,  6, 0, 1)),
            (100, _gen_coins, ("PP",  1, 20, 0, 1)),
         ],
    'Q': [
            (100, _gen_gems,  (1, 4, 0, 1)),
         ],
    'R': [
            (100, _gen_coins, ("GP",  2, 10, 0, 1)),
            (100, _gen_coins, ("PP", 10,  6, 0, 1)),
            (100, _gen_gems,  (2, 4, 0, 1)),
            (100, _gen_art,   (1, 3, 0, 1)),
         ],
    'S': [
            (100, _gen_magic, ("Potion", 1, 8, 0, 1)),
         ],
    'T': [
            (100, _gen_magic, ("Scroll", 1, 4, 0, 1)),
         ],
    'U': [
            ( 90, _gen_gems,  (2, 8, 0, 1)),
            ( 80, _gen_art,   (1, 6, 0, 1)),
            ( 70, _gen_magic, ("Any", 0, 0, 1, 1)),
         ],
    'V': [
            (100, _gen_magic, ("Any", 0, 0, 2, 1)),
         ],
    'W': [
            (100, _gen_coins, ("GP",  5,  6, 0,  1)),
            (100, _gen_coins, ("PP",  1,  8, 0,  1)),
            ( 60, _gen_gems,  (2, 8, 0, 1)),
            ( 50, _gen_art,   (1, 8, 0, 1)),
            ( 60, _gen_magic, ("Any", 0, 0, 2, 1)),
         ],
    'X': [
            (100, _gen_magic, ("Potion", 0, 0, 2, 1)),
         ],
    'Y': [
            (100, _gen_coins, ("GP",  2,  6, 0,  100)),
         ],
    'Z': [
            (100, _gen_coins, ("CP", 10,  3, 0,  10)),
            (100, _gen_coins, ("SP", 10,  4, 0,  10)),
            (100, _gen_coins, ("GP", 10,  6, 0,  10)),
            (100, _gen_coins, ("PP", 10,  4, 0,  10)),
            ( 55, _gen_gems,  (1, 6, 0, 1)),
            ( 50, _gen_art,   (2, 6, 0, 1)),
            ( 50, _gen_magic, ("Any", 0, 0, 3, 1)),
         ],

    # special classes

    'ART':    [ (100, _gen_art,   (0, 0, 1, 1)), ],
    'GEM':    [ (100, _gen_gems,  (0, 0, 1, 1)), ],
    'MAGIC':  [ (100, _gen_magic, ("Any", 0, 0, 1, 1)), ],
    'POTION': [ (100, _gen_magic, ("Potion", 0, 0, 1, 1)), ],
    'SCROLL': [ (100, _gen_magic, ("Scroll", 0, 0, 1, 1)), ],
    'RING':   [ (100, _gen_magic, ("Ring", 0, 0, 1, 1)), ],
    'WAND':   [ (100, _gen_magic, ("Wand", 0, 0, 1, 1)), ],
    'ROD':    [ (100, _gen_magic, ("Rod", 0, 0, 1, 1)), ],
    'STAFF':  [ (100, _gen_magic, ("Staff", 0, 0, 1, 1)), ],
    'MISC':   [ (100, _gen_magic, ("Misc", 0, 0, 1, 1)), ],
    'ARMOR':  [ (100, _gen_magic, ("Armor", 0, 0, 1, 1)), ],
    'WEAPON': [ (100, _gen_magic, ("Weapon", 0, 0, 1, 1)), ],
}

def Treasure(typ):
    tbl = _treasure_table[string.upper(typ)]
    tr = []
    for i in tbl:
        if Dice.D(1, 100, 0) <= i[0]:
            tr = tr + i[1](i[2])
    return tr

if __name__ == "__main__":
    import sys

    if len(sys.argv) < 2:
        print "Usage:  Treasure.py treasuretype [ treasuretype ... ]"
        sys.exit(0)

    for i in sys.argv[1:]:
        print "Treasure Type " + string.upper(i)
        tr = Treasure(i)
        vtot = 0.0
        ocat = ''
        for t in tr:
            if t.cat != ocat:
                print t.cat
            ocat = t.cat
            print "   ", t.qty, t.name, t.value, "GP ea.", \
                t.value * t.qty, "GP total"
            for i in t.desc:
                print "       ", i
            vtot = vtot + (t.qty * t.value)
        print "----- Total Value", vtot, "GP\n"

# end of script.
